function [Y,RRI]=berger(RRI,Fs)
% Resampling with the Berger algorithm
% [HRV,RRI] = berger(RRI, Fs)
% [HRV,RRI] = berger(ONSET, Fs)
% 
% RRI 	R-to-R interval 
% ONSET onset time QRS-complex
% Fs	target sampling rate
% HRV 	heart rate variability sampled with Fs
% RRI	R-to-R interval sampled with Fs
%

%       $Id: berger.m,v 1.2 2005/07/19 10:40:31 schloegl Exp $
%	Copyright (c) 1997-2005 by Alois Schloegl <a.schloegl@ieee.org>	
%    	This is part of the BIOSIG-toolbox http://biosig.sf.net/

% This library is free software; you can redistribute it and/or
% modify it under the terms of the GNU Library General Public
% License as published by the Free Software Foundation; either
% Version 2 of the License, or (at your option) any later version.
%
% This library is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% Library General Public License for more details.
%
% You should have received a copy of the GNU Library General Public
% License along with this library; if not, write to the
% Free Software Foundation, Inc., 59 Temple Place - Suite 330,
% Boston, MA  02111-1307, USA.

if isstruct(RRI)
        Fs0 = NaN; 
        if isfield(RRI,'EVENT')
                EVENT = RRI.EVENT; 
        else
                EVENT = RRI; 
        end;
        if isfield(EVENT,'SampleRate')
                Fs0 = EVENT.SampleRate; 
        elseif isfield(RRI.SampleRate)
                Fs0 = RRI.EVENT.SampleRate; 
        else    
                warning('Invalid input argument causes unknown source sampleing rate.');
        end;
        if isfield(EVENT,'POS') & isfield(EVENT,'TYP') & isfield(EVENT,'CHN') & isfield(EVENT,'DUR');
                ix = find(EVENT.TYP==hex2dec('0501'));
                if all(EVENT.CHN(ix(1)) == EVENT.CHN(ix));
                        on = EVENT.POS(EVENT.TYP==hex2dec('0501'))/Fs0;
                end;
        elseif isfield(EVENT,'POS') & isfield(EVENT,'TYP');
                on = EVENT.POS(EVENT.TYP==hex2dec('0501'))/Fs0;
        end;
        
elseif ~any(diff(RRI)<0),	
        on = RRI(:);%-min(RRI);
else		% calculate onset of bursts 
        on = cumsum([0;RRI(:)]);
end;

if ~exist('on','var')
        error('Invalid input argument'); 
end;        
%T=1/Fs;

N  = ceil(max(on)*Fs);
Y  = zeros(N,1);

on = [on; NaN] * Fs;

IX = 1;
for n = 1:N,
	while (on(IX+1)<(n-1)), IX=IX+1; end;
        ix = IX+1;
        CONT = 1;
        while CONT,
               
		% 1st case: RR interval is totally encompassed by sample window
	        if 0, 
		elseif n<=on(1)
                        Y(n) = nan;		
		elseif (on(ix-1) >= (n-1)) & (on(ix) <= (n+1)),
                        Y(n) = Y(n) + 1;
		% 2nd case: RR interval partially overlaps beginning of sample window
	        elseif (on(ix-1) <  (n-1)) & (on(ix) <  (n+1)),
                        Y(n) = Y(n) + (on(ix)-(n-1))/(on(ix)-on(ix-1));
		% 3rd case: RR interval partially overlaps the end of sample window
	        elseif (on(ix-1) >  (n-1)) & (on(ix) >  (n+1)),
                        Y(n) = Y(n) + ((n+1) - on(ix-1))/(on(ix)-on(ix-1));
		% 4th case: RR interval totally overlaps the sample window
	        elseif (on(ix-1) <= (n-1)) & (on(ix) >= (n+1)),
                        Y(n) = Y(n) + 2/(on(ix)-on(ix-1));
		elseif isnan(on(ix)),
                        Y(n) = nan;
                else
                        disp([ix,on(ix-1),on(ix), n, (n+1),Y(n)])
                        fprintf(2,'Warning BERGER: Invalid program state');
                        Y(n) = nan;
                end;
                if (on(ix)<(n+1)),%(on(ix-1)<(n+1)*T), 
                        ix = ix+1;
                else
                        CONT=0;
                end;
        end;
end;        

RRI = (2*max(on)/N/Fs)./Y;
%Y=Fs/2*N/max(on(1:length(on)-1))*Y;
Y = 60./RRI;


